package de.intevation.flys.artifacts;

import java.util.List;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import com.vividsolutions.jts.geom.Envelope;

import de.intevation.artifacts.ArtifactFactory;
import de.intevation.artifacts.CallMeta;

import de.intevation.artifactdatabase.state.DefaultOutput;
import de.intevation.artifactdatabase.state.Facet;
import de.intevation.artifactdatabase.state.State;

import de.intevation.flys.model.River;
import de.intevation.flys.model.Floodmaps;

import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.RiverFactory;
import de.intevation.flys.artifacts.resources.Resources;
import de.intevation.flys.utils.FLYSUtils;


public class WMSFloodmapsArtifact extends WMSDBArtifact {

    public static final String NAME = "floodmaps";


    private static final Logger logger =
        Logger.getLogger(WMSFloodmapsArtifact.class);


    @Override
    public void setup(
        String          identifier,
        ArtifactFactory factory,
        Object          context,
        CallMeta        callMeta,
        Document        data)
    {
        logger.debug("WMSFloodmapsArtifact.setup");

        super.setup(identifier, factory, context, callMeta, data);
    }


    @Override
    public String getName() {
        return NAME;
    }


    @Override
    public State getCurrentState(Object cc) {
        State s = new FloodmapsState(this);

        List<Facet> fs = facets.get(getCurrentStateId());

        DefaultOutput o = new DefaultOutput(
            "floodmap",
            "floodmap",
            "image/png",
            fs,
            "map");

        s.getOutputs().add(o);

        return s;
    }


    public static class FloodmapsState extends WMSDBState implements FacetTypes
    {
        private static final Logger logger =
            Logger.getLogger(FloodmapsState.class);

        protected int    riverId;
        protected String floodmapName;

        public FloodmapsState(WMSDBArtifact artifact) {
            super(artifact);
            riverId      = 0;
            floodmapName = null;
        }

        public int getRiverId() {
            if (riverId == 0) {
                String ids = artifact.getDataAsString("ids");

                try {
                    String[] parts = ids.split("_");
                    riverId = Integer.valueOf(parts[0]);
                }
                catch (NumberFormatException nfe) {
                    logger.error("Cannot parse river id from '" + ids + "'");
                }
            }

            return riverId;
        }

        public String getFloodmapName() {
            if (floodmapName == null) {
                String ids = artifact.getDataAsString("ids");

                String parts[] = ids.split("_");
                floodmapName   = parts[1];
            }

            return floodmapName;
        }

        @Override
        protected String getFacetType() {
            return FLOODMAP_FLOODMAPS;
        }

        @Override
        protected String getTitle(CallMeta meta) {
            return Resources.getMsg(
                meta,
                FLOODMAP_FLOODMAPS,
                FLOODMAP_FLOODMAPS);
        }

        @Override
        protected String getUrl() {
            return FLYSUtils.getUserWMSUrl(artifact.identifier());
        }

        @Override
        protected String getSrid() {
            River river = RiverFactory.getRiver(getRiverId());
            return FLYSUtils.getRiverSrid(river.getName());
        }

        @Override
        protected Envelope getExtent() {
            List<Floodmaps> floodmaps =
                Floodmaps.getFloodmaps(getRiverId(), getFloodmapName());

            Envelope max = null;

            for (Floodmaps f: floodmaps) {
                Envelope env = f.getGeom().getEnvelopeInternal();

                if (max == null) {
                    max = env;
                    continue;
                }

                max.expandToInclude(env);
            }

            return max;
        }

        @Override
        protected String getFilter() {
            return "river_id=" + String.valueOf(getRiverId())
                + "name=" + getFloodmapName();
        }

        @Override
        protected String getDataString() {
            String srid = getSrid();

            if (FLYSUtils.isUsingOracle()) {
                return "geom FROM floodmaps USING SRID " + srid;
            }
            else {
                return "geom FROM floodmaps USING UNIQUE id USING SRID " + srid;
            }
        }

        @Override
        protected String getGeometryType() {
            return "MULTIPOLYGON";
        }
    } // end of WMSKmState
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
